package com.afp.ordermanagement.annotation;

import com.afp.ordermanagement.service.ManagerTokenVerifier;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.web.reactive.BindingContext;
import org.springframework.web.reactive.result.method.HandlerMethodArgumentResolver;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

public class AccessTokenResolver implements HandlerMethodArgumentResolver {
    @Autowired
    ManagerTokenVerifier managerTokenVerifier;

    @Override
    public boolean supportsParameter(MethodParameter methodParameter) {
        boolean hasClassAnnotation = methodParameter.getDeclaringClass().isAnnotationPresent(AuthManagerController.class);
        boolean hasMethodAnnotation = methodParameter.hasMethodAnnotation(AuthManagerResponse.class);
        boolean hasParameterAnnotation = methodParameter.hasParameterAnnotation(AccessToken.class);
        return (hasClassAnnotation || hasMethodAnnotation) && hasParameterAnnotation;
    }

    @Override
    public Mono<Object> resolveArgument(MethodParameter methodParameter, BindingContext bindingContext, ServerWebExchange serverWebExchange) {
        String accessToken = managerTokenVerifier.getTokenHeader(serverWebExchange);
        return Mono.just(accessToken);
    }
}
