package com.afp.ordermanagement.annotation;

import com.afp.ordermanagement.model.Manager;
import com.afp.ordermanagement.service.ManagerTokenVerifier;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.web.reactive.BindingContext;
import org.springframework.web.reactive.result.method.HandlerMethodArgumentResolver;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.lang.reflect.Field;
import java.util.Arrays;
import java.util.List;

public class ManagerPayloadResolver implements HandlerMethodArgumentResolver {
    @Autowired
    ManagerTokenVerifier managerTokenVerifier;

    @Override
    public boolean supportsParameter(MethodParameter methodParameter) {
        boolean hasClassAnnotation = methodParameter.getDeclaringClass().isAnnotationPresent(AuthManagerController.class);
        boolean hasMethodAnnotation = methodParameter.hasMethodAnnotation(AuthManagerResponse.class);
        boolean hasParameterAnnotation = methodParameter.hasParameterAnnotation(ManagerPayload.class);
        return (hasClassAnnotation || hasMethodAnnotation) && hasParameterAnnotation;
    }

    @Override
    public Mono<Object> resolveArgument(MethodParameter methodParameter, BindingContext bindingContext, ServerWebExchange serverWebExchange) {
        String accessToken = managerTokenVerifier.getTokenHeader(serverWebExchange);

        ManagerPayload annotation = methodParameter.getParameterAnnotation(ManagerPayload.class);

        String value = annotation.value(), pluck = annotation.pluck(), selectedField;

        List<Field> declaredFields = Arrays.asList(Manager.class.getDeclaredFields());

        Manager manager = managerTokenVerifier.createManagerFromToken(accessToken);

        if (value.isEmpty() && pluck.isEmpty()) {
            return Mono.just(manager);
        } else {
            selectedField = value.isEmpty() ? pluck : value;

            if (!declaredFields.contains(selectedField))
                return Mono.just(manager);

            try {
                Field field = Manager.class.getDeclaredField(selectedField);
                field.setAccessible(true);

                return Mono.just(field.get(manager));
            } catch (Exception ignore) {
                return Mono.just(manager);
            }
        }
    }
}