package com.afp.ordermanagement.controller;

import com.afp.ordermanagement.annotation.AuthManagerController;
import com.afp.ordermanagement.annotation.ManagerPayload;
import com.afp.ordermanagement.exception.ResourceNotFoundException;
import com.afp.ordermanagement.model.Manager;
import com.afp.ordermanagement.service.ManagerService;
import io.swagger.v3.oas.annotations.parameters.RequestBody;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.validation.Valid;

@Validated
@RestController
@AuthManagerController
@RequestMapping("/api/managers/")
public class ManagerController {
    @Autowired
    ManagerService managerService;

    @PostMapping("/auth")
    public ResponseEntity<Mono<Manager>> signUpOrLogInManager(@ManagerPayload Manager managerPayload) {
        Mono<Manager> manager = managerService.getByEmail(managerPayload.getEmail())
                .switchIfEmpty(managerService.create(managerPayload));
        return ResponseEntity.ok(manager);
    }

    @GetMapping("/account")
    public ResponseEntity<Mono<Manager>> getManagerDetails(@ManagerPayload Manager managerPayload) {
        Mono<Manager> existingManager = managerService.getByEmail(managerPayload.getEmail())
                .switchIfEmpty(Mono.error(new ResourceNotFoundException()));
        return ResponseEntity.ok(existingManager);
    }

    @PatchMapping("/account")
    public ResponseEntity<Mono<Manager>> updateManagerDetails(@ManagerPayload Manager managerPayload, @Valid @RequestBody Manager managerBody) {
        return ResponseEntity.ok(managerService.updateByEmail(managerPayload.getEmail(), managerBody));
    }

    @DeleteMapping("/account")
    public ResponseEntity deleteManagerDetails(@ManagerPayload Manager managerPayload) {
        managerService.getByEmail(managerPayload.getEmail())
                .switchIfEmpty(Mono.error(new ResourceNotFoundException()))
                .flatMap(managerService::delete);
        return ResponseEntity.noContent().build();
    }
}
