package com.afp.ordermanagement.controller;

import com.afp.ordermanagement.model.Order;
import com.afp.ordermanagement.reactivekafkaservice.Sender;
import com.afp.ordermanagement.service.OrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("/api")
public class OrderController {

    @Autowired
    private OrderService orderService;

    @Autowired
    Sender sender;


    /**
     * DESC  -   This route will let order manager get order status from warehouse
     * @param orderId
     */
    @GetMapping("/orderStatus/{orderId}")
    public void getOrderStatusFromWarehouse(@PathVariable String orderId) {
        sender.sendOrderIdToWarehouse(orderId);
    }

    @GetMapping("/orders")
    @CrossOrigin
    public Flux<Order> getAllOrders(){
        return orderService.getAllOrders();
    }

    @GetMapping("/orders/{orderId}")
    @CrossOrigin
    public Mono<Order> getOrderById(@PathVariable("orderId") String orderId) {
        return orderService.getOrderById(orderId);
    }

    @GetMapping("/orders/byCustomer/{customerId}")
    public Flux<Order> getAllOrdersByCustomerId(@PathVariable("customerId") String customerId) {
        return orderService.getAllOrdersByCustomerId(customerId);
    }

    @PostMapping("/orders")
    @ResponseStatus(HttpStatus.CREATED)
    public Mono<Order> saveOrder(@RequestBody Order order){

        return orderService.createOrder(order);
    }

    @PutMapping("/order/{orderId}")
    public Mono<ResponseEntity<Order>> updateOrder(@PathVariable(value = "orderId") String orderId, @RequestBody Order order){
        return orderService.updateOrderByOrderId(orderId, order)
                .map(updatedOrder -> ResponseEntity.ok(updatedOrder))
                .defaultIfEmpty(ResponseEntity.notFound().build());
    }


}
