package com.afp.ordermanagement.exception;

import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.support.WebExchangeBindException;
import org.springframework.web.server.ServerWebExchange;

@ControllerAdvice
@RequiredArgsConstructor
public class ControllerExceptionAdvice {
    @ExceptionHandler(BadAccessTokenException.class)
    public ResponseEntity<ErrorResponse> handleBadAccessTokenException(RuntimeException exc, ServerWebExchange exchange) {
        final HttpStatus status = HttpStatus.UNAUTHORIZED;
        final ServerHttpRequest request = exchange.getRequest();
        return new ResponseEntity<>(new ErrorResponse(
                status.value(),
                request.getPath().value(),
                status.getReasonPhrase(),
                exc.getMessage()
        ), status);
    }

    @ExceptionHandler(ResourceNotFoundException.class)
    public ResponseEntity<ErrorResponse> handleEntityNotFoundException(RuntimeException exc, ServerWebExchange exchange) {
        final HttpStatus status = HttpStatus.NOT_FOUND;
        final ServerHttpRequest request = exchange.getRequest();
        return new ResponseEntity<>(new ErrorResponse(
                status.value(),
                request.getPath().value(),
                status.getReasonPhrase(),
                exc.getMessage()
        ), status);
    }

    @ExceptionHandler(RuntimeException.class)
    public ResponseEntity<ErrorResponse> handleRuntimeException(RuntimeException exc, ServerWebExchange exchange) {
        final HttpStatus status = HttpStatus.INTERNAL_SERVER_ERROR;
        final ServerHttpRequest request = exchange.getRequest();
        return new ResponseEntity<>(new ErrorResponse(
                status.value(),
                request.getPath().value(),
                status.getReasonPhrase(),
                exc.getMessage()
        ), status);
    }

    @ExceptionHandler(WebExchangeBindException.class)
    public ResponseEntity<ErrorResponse> webExchangeBindException(WebExchangeBindException exc, ServerWebExchange exchange) {
        final HttpStatus status = exc.getStatus();
        final ServerHttpRequest request = exchange.getRequest();

        if (InvalidEntityResponse.isEntityValid(exc.getTarget())) {
            return new ResponseEntity<>(new ErrorResponse(
                    status.value(),
                    request.getPath().value(),
                    status.getReasonPhrase(),
                    exc.getMessage()
            ), status);
        }

        return new ResponseEntity<>(new InvalidEntityResponse(
                status.value(),
                request.getPath().value(),
                status.getReasonPhrase(),
                "Validation failed",
                exc.getTarget()
        ), status);
    }
}

