package com.afp.ordermanagement.seeder;

import com.afp.ordermanagement.model.Manager;
import com.afp.ordermanagement.repository.ManagerRepository;
import com.github.javafaker.Faker;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.core.env.Environment;
import org.springframework.core.env.Profiles;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.stream.IntStream;

@Component
public class ManagerSeeder {
    @Autowired
    ManagerRepository managerRepository;

    @Autowired
    Faker faker;

    @Autowired
    Environment env;

    // Verify that the current environment is NOT production or test
    boolean isEnvDevelopment() {
        return env.acceptsProfiles(Profiles.of("default", "dev", "development", "local"));
    }

    // Number of seed documents generated
    static private final int SEED_COUNT = 10;

    // Create manager seeds as soon as app is up and running
    @EventListener
    public void seedManagers(ContextRefreshedEvent event) {
        // Only generate seeds if in development environment
        if (isEnvDevelopment()) {
            managerRepository
                    .deleteAll()
                    .subscribe();
            IntStream.range(0, SEED_COUNT + 1).forEach(n -> {
                managerRepository
                        .insert(generateManager())
                        .subscribe();
            });
        }
    }

    private Manager generateManager(){
        Manager manager = new Manager();
        manager.setFirstName(faker.name().firstName());
        manager.setLastName(faker.name().lastName());
        manager.setEmail(faker.internet().emailAddress());
        manager.setGoogleId(faker.number().digits(21));
        manager.setImageUrl("https://picsum.photos/200/200");
        return manager;
    }
}
