package com.afp.ordermanagement.service;

import com.afp.ordermanagement.model.Order;
import com.afp.ordermanagement.model.OrderStatus;
import com.afp.ordermanagement.repository.OrderRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class OrderService {

    @Autowired
    OrderRepository orderRepository;

    public Mono<Order> createOrder(Order newOrder){
        String defaultOrderTrackingCode = "N/A";
        OrderStatus defaultOrderStatus = OrderStatus.RECEIVED;
        long serviceSystemTime = System.currentTimeMillis();
        newOrder.setOrderStatus(defaultOrderStatus);
        newOrder.setOrderTrackingCode(defaultOrderTrackingCode);
        newOrder.setOrderCreatedAt(serviceSystemTime);
        newOrder.setOrderUpdatedAt(serviceSystemTime);
        return orderRepository.save(newOrder);
    }

    public Flux<Order> getAllOrders(){
        return orderRepository.findAll();
    }

    public Mono<Order> getOrderById(String orderId) {
        return orderRepository.findById(orderId);
    }

    public Flux<Order> getAllOrdersByCustomerId(String customerId){
        return orderRepository.findByCustomerId(customerId);
    }

    public Mono<Order> updateOrderByOrderId(String orderId, Order newOrder){
        return orderRepository.findById(orderId)
                .flatMap(existingOrder -> {
                    existingOrder.setCustomerAddress(newOrder.getCustomerAddress());
                    existingOrder.setCustomerEmailAddress(newOrder.getCustomerEmailAddress());
                    existingOrder.setOrderTrackingCode(newOrder.getOrderTrackingCode());
                    existingOrder.setOrderItems(newOrder.getOrderItems());
                    existingOrder.setOrderStatus(newOrder.getOrderStatus());
                    return orderRepository.save(existingOrder);
                });
    }
}
